@@ -60,7 +60,7 @@ You can implement other color schemes easily, the syntax is fairly
 self-explanatory. Please send back new schemes you develop to the author for
 possible inclusion in future releases.
 
-$Id: ultraTB.py 1956 2006-11-30 05:22:31Z fperez $"""
+$Id: ultraTB.py 2156 2007-03-19 02:32:19Z fperez $"""
 
 #*****************************************************************************
 #       Copyright (C) 2001 Nathaniel Gray <n8gray@caltech.edu>
@@ -90,7 +90,7 @@ import types
 
 # IPython's own modules
 # Modified pdb which doesn't damage IPython's readline handling
-from IPython import Debugger
+from IPython import Debugger, PyColorize
 from IPython.ipstruct import Struct
 from IPython.excolors import ExceptionColors
 from IPython.genutils import Term,uniq_stable,error,info
@@ -99,6 +99,12 @@ from IPython.genutils import Term,uniq_stable,error,info
 # amount of space to put line numbers before verbose tracebacks
 INDENT_SIZE = 8
 
+# Default color scheme.  This is used, for example, by the traceback
+# formatter.  When running in an actual IPython instance, the user's rc.colors
+# value is used, but havinga module global makes this functionality available
+# to users of ultraTB who are NOT running inside ipython.
+DEFAULT_SCHEME = 'NoColor'
+
 #---------------------------------------------------------------------------
 # Code begins
 
@@ -151,11 +157,26 @@ def _fixed_getinnerframes(etb, context=1,tb_offset=0):
 # functionality to produce a pseudo verbose TB for SyntaxErrors, so that they
 # can be recognized properly by ipython.el's py-traceback-line-re
 # (SyntaxErrors have to be treated specially because they have no traceback)
-def _formatTracebackLines(lnum, index, lines, Colors, lvals=None):
+
+_parser = PyColorize.Parser()
+    
+def _formatTracebackLines(lnum, index, lines, Colors, lvals=None,scheme=None):
     numbers_width = INDENT_SIZE - 1
     res = []
     i = lnum - index
+
+    # This lets us get fully syntax-highlighted tracebacks.
+    if scheme is None:
+        try:
+            scheme = __IPYTHON__.rc.colors
+        except:
+            scheme = DEFAULT_SCHEME
+    _line_format = _parser.format2
+
     for line in lines:
+        new_line, err = _line_format(line,'str',scheme)
+        if not err: line = new_line
+        
         if i == lnum:
             # This is the line with the error
             pad = numbers_width - len(str(i))
@@ -181,6 +202,7 @@ def _formatTracebackLines(lnum, index, lines, Colors, lvals=None):
         i = i + 1
     return res
 
+
 #---------------------------------------------------------------------------
 # Module classes
 class TBTools:
@@ -248,6 +270,8 @@ class ListTB(TBTools):
         TBTools.__init__(self,color_scheme = color_scheme,call_pdb=0)
         
     def __call__(self, etype, value, elist):
+        Term.cout.flush()
+        Term.cerr.flush()
         print >> Term.cerr, self.text(etype,value,elist)
 
     def text(self,etype, value, elist,context=5):
@@ -397,6 +421,7 @@ class VerboseTB(TBTools):
         # some locals
         Colors        = self.Colors   # just a shorthand + quicker name lookup
         ColorsNormal  = Colors.Normal  # used a lot
+        col_scheme    = self.color_scheme_table.active_scheme_name
         indent        = ' '*INDENT_SIZE
         exc           = '%s%s%s' % (Colors.excName, str(etype), ColorsNormal)
         em_normal     = '%s\n%s%s' % (Colors.valEm, indent,ColorsNormal)
@@ -635,7 +660,8 @@ class VerboseTB(TBTools):
                 frames.append(level)
             else:
                 frames.append('%s%s' % (level,''.join(
-                    _formatTracebackLines(lnum,index,lines,self.Colors,lvals))))
+                    _formatTracebackLines(lnum,index,lines,Colors,lvals,
+                                          col_scheme))))
 
         # Get (safely) a string form of the exception info
         try:
@@ -716,6 +742,8 @@ class VerboseTB(TBTools):
     def handler(self, info=None):
         (etype, evalue, etb) = info or sys.exc_info()
         self.tb = etb
+        Term.cout.flush()
+        Term.cerr.flush()
         print >> Term.cerr, self.text(etype, evalue, etb)
 
     # Changed so an instance can just be called as VerboseTB_inst() and print
@@ -833,6 +861,8 @@ class AutoFormattedTB(FormattedTB):
         
         if out is None:
             out = Term.cerr
+        Term.cout.flush()
+        out.flush()
         if tb_offset is not None:
             tb_offset, self.tb_offset = self.tb_offset, tb_offset
             print >> out, self.text(etype, evalue, etb)
